<?php
/* ============= Cron Relay - by LuxSoft (www.luxsoft.eu) =============

This script should be started via a cron job once a day or each hour.
It will subsequently . . . 
- start the specified scripts at the specified hour
and optionally . . .
- send a notification mail to the recipient (see below) 
- log the result in the file cron.log
When the size of the cron.log file exceeds 100kb, the oldest half of 
the file will be flushed.
If run once a day, the hour in SCRIPTS TO BE RUN VIA CRON should be 'x'

IMPORTANT: THIS SCRIPT REQUIRES THE PHP CURL EXTENSION!
*/

/*====================== CONFIGURATION SECTION ======================*/

/*------------------- SPECIFY BOTH EMAIL ADDRESSES ------------------*/
$from = "tom-dowd@fastmail.com"; //originator
$to   = "tom-dowd@fastmail.com"; //recipient (may be same as originator)

/*-------------------------- YOUR TIME ZONE -------------------------*/
$tZone = "Europe/Amsterdam"; //see www.php.net/manual/en/timezones.php

/*-------------------- SCRIPTS TO BE RUN VIA CRON -------------------*/
$scripts = array (
/*
for each script add an array in the format: ["u","h","o"], where 
u: URL of the script to start
h: hour when to start the script ("0" - "23", or "x": each time)
o: options ("e": send confirmation email, "l": log result, "el": both)
*/
["https://www.mywebsite.com/calendar/testdata.php","x","l"],
["https://www.mysoftware.eu/mycron.php","3","el"],
["https://kal.mywebpage.eu/crontorun.php","3","el"],
);

/*----------------- ADD SERVER AND PHP DATA TO EMAIL ----------------*/
$addSnP = "n"; //"n": no, "y": yes

/*=================== END OF CONFIGURATION SECTION ==================*/

//initialize
date_default_timezone_set($tZone);
$datetime = date("Y-m-d H:i");
$time = date("H:i");

//run specified scripts
if (extension_loaded('curl')) { //cURL extension enabled
	$hour = date('G');
	$emlMsg = $logMsg = '';
	$ch = curl_init();
	curl_setopt($ch,CURLOPT_HEADER, 0);
	foreach ($scripts as $script) {
		if (strtolower($script[1]) == 'x' or strpos("~;$script[1];",";$hour;")) {
			curl_setopt($ch,CURLOPT_URL,$script[0]);
			$result = curl_exec($ch) ? 'OK' : 'Failed';
			if (stripos('~'.$script[2],"e")) { $emlMsg .= "• {$script[0]}: {$result}\n"; }
			if (stripos('~'.$script[2],"l")) { $logMsg .= "• {$script[0]}: {$result}\n"; }
		}
	}
	curl_close($ch);
	if ($emlMsg) { $emlMsg = "Scripts started @ $time:\n".$emlMsg; }
	if ($logMsg) { $logMsg = "Scripts started @ $time:\n".$logMsg; }
} else {
	$emlMsg = $logMsg = "ERROR: THIS SCRIPT REQUIRES THE PHP CURL EXTENSION!\n";
}

if ($emlMsg) {//send confirmation email
	$subject = "Message from your server - triggered by cron";
	$message = "Hello, this email comes from your cron.php script.\n
The cronjob ran at {$datetime} (server time) from directory ".getcwd()."
\n{$emlMsg}";
if (strtolower($addSnP) == "y") {
	$message .= "\nServer info:
• Host name: ".gethostname()."
• OS: ".PHP_OS."
\nPHP info:
• Version: ".PHP_VERSION."
• PDO-SQLite extension: ".(extension_loaded('pdo_sqlite') ? 'Yes' : 'No ')."
• PDO-MySQL extension: ".(extension_loaded('pdo_mysql') ? 'Yes' : 'No ')."\n";
}
	$message .= "\n~ Made by LuxSoft ~";
	$headers = "From: {$from}\r\nReply-To: {$from}\r\nReturn-Path: {$from}\r\n";
	$result = mail($to,$subject,$message,$headers) ? 'Sent' : 'Failed'; //send email
}

if ($logMsg) { //log result
	if (file_exists('cron.log') and filesize('cron.log') > 100000) { //flush oldest half
		$fileArray = file('cron.log');
		$fileArray = array_slice ($fileArray,intval(count($fileArray)/2));
		file_put_contents ('cron.log',implode('',$fileArray));
	}
	$email = $emlMsg ? " - Mail to {$to}: {$result}" : "";
	file_put_contents ('cron.log',"\n{$datetime} Cron run{$email}\n{$logMsg}",FILE_APPEND);
}
?>