<?php
/*
===== LUXLOG - LOGGING VISITORS DATA ON YOUR SERVER =====
======== Roel Buining - LuxSoft - www.LuxSoft.eu ========
*/

//======================= settings ======================
$logFolder = './logs'; //log file folder
$logItems = '12345678'; //items and order to log - 1: web page, 2: view port, 3: IP-address, 4: country, 5: city, 6: browser, 7: platform, 8: device type
$maxSize = 20000; //maximum log file size (bytes). If exceeded, the file will be timestamped and a new file will be started
$timeZone = 'Europe/Amsterdam'; //see https://www.php.net/manual/en/timezones.php
//=======================================================

//init
$ipInfoUrl = 'https://ipinfo.io/$1'; //service for IP address data ($1 will be replaced by the IP address)
if (!is_dir($logFolder)) { mkdir($logFolder); }
$titles = [0 => 'Date & Time', 1 => 'Web Page', 2 => 'View Port', 3 => 'IP-address', 4 => 'Country', 5 => 'City', 6 => 'Browser', 7 => 'Platform', 8 => 'Device'];

$data = json_decode(rawurldecode(trim(file_get_contents("php://input")))); //read and decode JSON input

date_default_timezone_set($timeZone);

//collect all data items

/* general data */
$logFile = $data->lfName;
$agent = '-'.strtolower($data->agent).'-'; //user agent (browser)
$items[1] = $data->pName;
$items[2] = $data->vpWidth.' x '.$data->vpHeight;

/* user IP address data */
$items[3] = $_SERVER['REMOTE_ADDR'];
$url = str_replace('$1',$items[3],$ipInfoUrl);
$ipData = json_decode(file_get_contents($url)); //object with details
$items[4] = $ipData->country ?? 'Unknown'; //country name
$items[5] = $ipData->city ?? 'Unknown'; //city name

//parse user agent (browser)

/* browser */
if (strpos($agent,'chrome')) {
  $items[6] = 'Chrome'; 
} elseif (strpos($agent,'firefox')) {
  $items[6] = 'Firefox'; 
} elseif (strpos($agent,'opera')) {
  $items[6] = 'Opera'; 
} elseif (strpos($agent,'edg')) {
  $items[6] = 'Edge'; 
} elseif (strpos($agent,'safari')) {
  $items[6] = 'Safari'; 
} else {
  $items[6] = 'unknown'; 
}

/* platform */
if (strpos($agent,'windows')) {
  $items[7] = 'Windows'; 
} elseif (strpos($agent,'android')) {
  $items[7] = 'Android'; 
} elseif (strpos($agent,'iphone') or strpos($agent,'ipad')) {
  $items[7] = 'iOS'; 
} else {
  $items[7] = 'unknown'; 
}

/* device type */
$isMob = strpos($agent,'mobile'); //check if mobile
$isTab = strpos($agent,'tablet'); //check if tablet
if($isMob){
	$items[8] = $isTab ? 'Tablet' : 'Mobile';
}else{ 
	$items[8] = 'Desktop'; 
}

//now we have collected all required data

//log data
$lfName = rtrim($logFolder,'/').'/'.$logFile.'.log';
if (file_exists($lfName) and filesize($lfName) > $maxSize) { //max. file size reached
	$newName = substr($lfName,0,-4).'_'.date('Ymd@H:i').substr($lfName,-4);
	rename($lfName,$newName);
}
if (!file_exists($lfName)) { //start new file
	$logTitles = $titles[0]; //init
	foreach (str_split($logItems) as $i) {
		$logTitles .= ', '.$titles[$i];
	}
	file_put_contents($lfName,$logTitles,LOCK_EX);
}
//add data to the log file
$logData = "\n".date('Y.m.d H:i:s'); //init
foreach (str_split($logItems) as $i) {
	$logData .= ', '.$items[$i];
}
file_put_contents($lfName,$logData,FILE_APPEND | LOCK_EX);

echo json_encode(["ok"=>"OK","response"=>"Data logged"]);
?>
